﻿//-----------------------------------------------------------------------------
// Game1.cs
//
// Kyoung Shin Park @ Dankook University (4/4/2011)
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;


namespace Primitives3D
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont spriteFont;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;

        // Store a list of primitive models, plus which one is currently selected.
        List<GeometricPrimitive> primitives = new List<GeometricPrimitive>();
        float angle = 0.0f;
        Vector3 cameraPosition = new Vector3(0, 0, 2.5f); // RHS

        // store a wireframe rasterize state
        RasterizerState wireFrameState;

        // Are we rendering in wireframe mode?
        bool isWireframe = true;
        bool isAnimation = true;

        float xangle = 0.0f;    // => 30.0f
        float yangle = 0.0f;    // => 60.0f
        float zangle = 0.0f;    // => 45.0f
        enum AngleMode { None, Yaw, Pitch, Roll };
        AngleMode mode = AngleMode.None;

        Matrix tx = Matrix.CreateTranslation(2.0f, 0.0f, 0.0f); // RHS
        Matrix rx = Matrix.CreateRotationX(MathHelper.ToRadians(30.0f)); // RHS
        Matrix ry = Matrix.CreateRotationY(MathHelper.ToRadians(60.0f)); // RHS
        Matrix rz = Matrix.CreateRotationZ(MathHelper.ToRadians(45.0f)); // RHS
        Matrix ra = Matrix.CreateFromAxisAngle(new Vector3(1.0f, 1.0f, 1.0f), MathHelper.ToRadians(45.0f)); // RHS
        Matrix sy = Matrix.CreateScale(1, 2, 1); // RHS
        Matrix tr, rt, trs, srt;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            System.Diagnostics.Trace.WriteLine(tx);
            System.Diagnostics.Trace.WriteLine(rx);
            System.Diagnostics.Trace.WriteLine(ry);
            System.Diagnostics.Trace.WriteLine(rz);
            System.Diagnostics.Trace.WriteLine(ra);
            System.Diagnostics.Trace.WriteLine(sy);

            tr = tx * rz; // translate, and then rotate (Row-Major Order)
            rt = rz * tx; // rotate, and then translate
            trs = tx * rz * sy; // translate, and then rotate, and then scale
            srt = sy * rz * tx; // scale, and then rotate, and then translate
            System.Diagnostics.Trace.WriteLine(tr);
            System.Diagnostics.Trace.WriteLine(rt);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            spriteFont = Content.Load<SpriteFont>("hudfont");

            // TODO: use this.Content to load your game content here
            primitives.Add(new CubePrimitive(GraphicsDevice));
            primitives.Add(new SpherePrimitive(GraphicsDevice));
            primitives.Add(new CylinderPrimitive(GraphicsDevice));
            primitives.Add(new TorusPrimitive(GraphicsDevice));
            primitives.Add(new PyramidPrimitive(GraphicsDevice));
            primitives.Add(new TeapotPrimitive(GraphicsDevice));

            wireFrameState = new RasterizerState()
            {
                FillMode = FillMode.WireFrame,
                CullMode = CullMode.None,
            };
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            HandleInput();

            base.Update(gameTime);
        }

        #region HandleInput
        protected void HandleInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            // Check for exit.
            if (IsPressed(Keys.Escape, Buttons.Back))
            {
                Exit();
            }

            // Toggle animation?
            if (IsPressed(Keys.A, Buttons.A))
            {
                isAnimation = !isAnimation;
            }

            // Toggle wireframe?
            if (IsPressed(Keys.W, Buttons.B))
            {
                isWireframe = !isWireframe;
            }

            // Change Y angle?
            if (IsPressed(Keys.X, Buttons.X))
            {
                angle += 5.0f;
            }

            // Change Camera Position?
            if (IsPressed(Keys.T, Buttons.Y))
            {
                cameraPosition.Z -= 0.1f;
            }
            if (IsPressed(Keys.Y, Buttons.Y))
            {
                cameraPosition.Z += 0.1f;
            }

            // Change rotation?
            if (IsPressed(Keys.Space, Buttons.Back))
            {
                mode++;
                if (mode > AngleMode.Roll) mode = AngleMode.None;
                System.Diagnostics.Trace.WriteLine("mode=" + mode);

                if (mode == AngleMode.None)
                {
                    xangle = yangle = zangle = 0.0f;
                }
                else if (mode == AngleMode.Yaw)
                {
                    yangle = 60.0f;
                }
                else if (mode == AngleMode.Pitch)
                {
                    xangle = 30.0f;
                }
                else if (mode == AngleMode.Roll)
                {
                    zangle = 45.0f;
                }
            }
        }

        /// <summary>
        /// Checks whether the specified key or button has been pressed.
        /// </summary>
        bool IsPressed(Keys key, Buttons button)
        {
            return (currentKeyboardState.IsKeyDown(key) &&
                    prevKeyboardState.IsKeyUp(key)) ||
                   (currentGamePadState.IsButtonDown(button) &&
                    prevGamePadState.IsButtonUp(button));
        }
        #endregion

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            //RenderState renderState = GraphicsDevice.RenderState;
            if (isWireframe)
            {
                GraphicsDevice.RasterizerState = wireFrameState;
            }
            else
            {
                GraphicsDevice.RasterizerState = RasterizerState.CullCounterClockwise;
            }

            // Create camera matrices, making the object spin.
            float time = (float)gameTime.TotalGameTime.TotalSeconds;
            if (isAnimation) angle = time * 30.0f;

            float aspect = GraphicsDevice.Viewport.AspectRatio;

            Matrix world = Matrix.Identity; 
            Matrix view = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
            Matrix projection = Matrix.CreatePerspectiveFieldOfView(1, aspect, 1, 10);
            primitives[5].Draw(world, view, projection, Color.Black);

            // Draw the teapot with x/y/z translate matrix, x/y/z rotation matrix, x/y/z scale matrix (RHS)
            //primitives[5].Draw(Matrix.CreateTranslation(1, 0, 0), view, projection, Color.Cyan); //X
            //primitives[5].Draw(Matrix.CreateTranslation(0, 1, 0), view, projection, Color.Magenta); //Y
            //primitives[5].Draw(Matrix.CreateTranslation(0, 0, 1), view, projection, Color.Yellow); //Z
            //primitives[5].Draw(r1, view, projection, Color.Cyan); //X
            //primitives[5].Draw(r2, view, projection, Color.Magenta); //Y
            //primitives[5].Draw(r3, view, projection, Color.Yellow); //Z
            //primitives[5].Draw(Matrix.CreateScale(2, 1, 1), view, projection, Color.Cyan); //X
            //primitives[5].Draw(Matrix.CreateScale(1, 2, 1), view, projection, Color.Magenta); //Y
            //primitives[5].Draw(Matrix.CreateScale(1, 1, 2), view, projection, Color.Yellow); //Z

            // Draw the teapot with different transformation compositions: T*R, R*T, T*R*S, S*R*T
            //primitives[5].Draw(tr, view, projection, Color.Red);
            //primitives[5].Draw(rt, view, projection, Color.Blue);
            //primitives[5].Draw(trs, view, projection, Color.Green);
            //primitives[5].Draw(srt, view, projection, Color.Yellow);

            // Draw the teapot with different orientation methods: YawPitchRoll vs Y*X*Z
            world = Matrix.CreateRotationY(MathHelper.ToRadians(yangle)) * Matrix.CreateRotationX(MathHelper.ToRadians(xangle)) * Matrix.CreateRotationZ(MathHelper.ToRadians(zangle)) * Matrix.CreateTranslation(-1.0f, 0.0f, 0.0f);
            primitives[5].Draw(world, view, projection, Color.Blue);
            world = Matrix.CreateFromYawPitchRoll(MathHelper.ToRadians(yangle), MathHelper.ToRadians(xangle), MathHelper.ToRadians(zangle)) * Matrix.CreateTranslation(1.0f, 0.0f, 0.0f);
            primitives[5].Draw(world, view, projection, Color.Red);
            
            // Reset the fill mode renderstate.
            GraphicsDevice.RasterizerState = RasterizerState.CullCounterClockwise;

            // Draw overlay text.
            string text = "A = Animation On/Off\n" +
                          "W = Wireframe On/Off\n" +
                          "X = Speed Rotation\n" +
                          "T/Y = Zoom In/Out\n" +
                          "Space = Change angle";

            spriteBatch.Begin();
            spriteBatch.DrawString(spriteFont, text, new Vector2(48, 48), Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
